//<c>   Copyright 1999 by Gerry J. Danen; all rights reserved.

//<t>   Convert an FTN-Compliant NodeList to a BBS Listing

/*      Usage:  NL2BLIST file_in file_out [zone]

        Full pathnames are supported for the filenames.

        Purpose:  To read a single nodelist and translate it from the raw St.
        Louis nodelist to a fixed file format:

        bbs name    location    phone number

        This program is based on FNI.C, but I've gone back to my own library functions.

        If you want to change this program, go ahead.  All I ask is that you
        e-mail the changed version to gdanen@bigfoot.com.
*/
#include"gd_tools.h"

#define DeBug 0

#define pgVERS_ "V1.10"
#define pgTITL_ "BBS List Creator"
#define pgCOPR_ "1999"


#define BUFSIZE 10000
UCHAR   iFile[FS_LEN], oFile[FS_LEN], work[BUFSIZE+2], *w, word[81],
        sZone[81], sHost[81], sNode[81], sZoneHostNode[81], argZone[81]="" ;
long    lDown=0, lHold=0, lHost=0, lHub=0, lPvt=0, lRegion=0, lZone=0, lRecs=0, lWritten=0 ;
int     iZHN=0, iBBS=0, iSysOp=0, iLocn=0, iPhone=0 ;
FILE    *iFh, *oFh ;


void    show_help_and_quit( void )
{
    logon(1);
    fprintf( stderr,
             "\n%s converts an FTN-Compliant Nodelist to a BBS listing in text format\n\n"
             "\tUsage:  %s file_in file_out [zone]\n\n"
             "If zone is specified, only that zone is included,\n"
             "otherwise all zones will go to the output file.\n\n",
             get_global_program_name(), get_global_program_name() );
    exit( GD_ERROR_HELP );
}


void    fix_underscore( UCHAR s[] )
{
    static int x ;

    LOOP(x, 0, ((int)strlen(s)), 1)             // change all "_" to spaces
    {
        if ( s[x] == '_' )
            s[x] = ' ' ;
    }
}


void    process_unknown( void )
{
    strupr( word );
    work[38] = EOS ;
    fprintf( stderr, "\rUnknown entry \"%s\" at line %-6ld-> %s\n", word, lRecs, work );
}


void    process_node_entry( void )
{
    static UCHAR sBBS[81], sSysOp[81], sLocn[81], sPhone[81] ;

    st_get_word( sBBS, work );                  // extract BBS name
    st_remove_word( work );                     // remove from buffer
    st_get_word( sLocn, work );                 // extract location
    st_remove_word( work );                     // remove from buffer
    st_get_word( sSysOp, work );                // extract SysOp name
    st_remove_word( work );                     // remove from buffer
    st_get_word( sPhone, work );                // extract phone number
    st_remove_word( work );                     // remove from buffer
    fix_underscore( sBBS   );
    fix_underscore( sLocn  );
    if ( st_eq(sPhone, "-unpublished-") )
        strcpy( sPhone, "" );

    if ( strlen(argZone) )
    {
        if ( ! st_eq(argZone, sZone) )
            return ;
    }

    if ( ! strlen(sPhone) )
        return ;

#if DeBug
    //  the field sizes collected here will determine the %-xx.xxs parts
    //  in the fprintf below.
    if ( (int)strlen(sBBS) > iBBS )
        iBBS = (int)strlen(sBBS);
    if ( (int)strlen(sLocn) > iLocn )
        iLocn = (int)strlen(sLocn);
    if ( (int)strlen(sPhone) > iPhone )
        iPhone = (int)strlen(sPhone);
#endif

    fprintf( oFh, "%-30.30s%-30.30s%s\n", sLocn, sBBS, sPhone );
    lWritten ++ ;
}


void    process_zone( void )
{
    st_get_word( sZone, work );                 // extract zone
    st_remove_word( work );                     // remove from buffer
    strcpy( sHost, sZone );                     // set host to same value
    strcpy( sNode, "0" );                       // set node to 0
    fprintf( stderr, "\nZone %s...\n", sZone );
}


void    process_host( void )
{
    st_get_word( sHost, work );                 // extract host
    st_remove_word( work );                     // remove from buffer
    strcpy( sNode, "0" );                       // set node to 0
}


void    process_node( void )
{
    st_get_word( sNode, work );                 // extract node number
    st_remove_word( work );                     // remove from buffer
    process_node_entry();
}


void    main( int argc, char *argv[] )
{
    int x ;

    tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
    logon(0);

    if ( argc < 3 )
        show_help_and_quit();

    st_cpy( iFile, (UCHAR *) argv[1], FS_LEN );
    st_cpy( oFile, (UCHAR *) argv[2], FS_LEN );
    strupr( (char *) iFile );
    strupr( (char *) oFile );
    if ( argc == 4 )
        st_cpy( argZone, (UCHAR *) argv[3], 81 );

    if ( ( iFh = fopen( iFile, "r" ) ) == NULL )
    {
        logon(1);
        fprintf( stderr, "\n\nFile %s not found\a\n", iFile );
        show_help_and_quit();
    }

    oFh = fl_create_or_replace_tty( oFh, oFile, "w", YES, NO );

    fprintf( stderr, "Processing node list %s for ", iFile );
    if ( strlen(argZone) )
        fprintf( stderr, "zone %s\n", argZone );
    else
        fprintf( stderr, "all zones\n" );

    while ( (w = fgets( work, BUFSIZE, iFh )) )
    {
        st_trim( work );                         // remove leading/trailing spaces
        if ( work[0] != ';' )                   // ignore comments
        {
            st_remove_spaces( work );           // remove embedded spaces
            lRecs ++ ;                          // count data records
            if ( ! (lRecs % 100L) )
                fprintf( stderr, "\r%ld", lRecs ); // show progress
            LOOP(x, 0, ((int)strlen(work)), 1)  // change all commas to spaces
            {
                if ( work[x] == ',' )
                    work[x] = ' ' ;
            }
            if ( work[0] == ' ' )               // first char _was_ a ","
            {
                process_node();
            }
            else                                // we might have a special entry
            {                                   //  .. Down, Hold, Host, Hub, Pvt, Region, Zone
                st_get_word( word, work );      // get first word in string
                st_remove_word( work );         // remove first word in string
                if ( st_eq(word, "zone") )
                {
                    lZone ++ ;
                    process_zone();
                }
            }
        }
    }
    _fcloseall();

    fprintf( stderr, "\r%ld records written\n\n", lWritten );

    fprintf( stderr,
             "If you want to change this program, go ahead.\n"
             "All I ask is that you e-mail me a copy of\n"
             "the changed source at gdanen@bigfoot.com.\n\n" );

#if DeBug
    fprintf( stderr,
             "\nDebug Information:\n"
             "\tlongest BBS name  %d\n"
             "\tlongest Location  %d\n"
             "\tlongest Phone #   %d\n", iBBS, iLocn, iPhone );
#endif

    exit( GD_ERROR_NONE );
}
